#include <joyos.h>
#include <math.h>

#define PI (3.14159)
#define max( a, b ) ( ((a) > (b)) ? (a) : (b) )
#define min( a, b ) ( ((a) < (b)) ? (a) : (b) )
#define angleDist(f, t) (makeCircular(t-f))
#define RAD_TO_DEGREE (57.2957795)


//an object to hold an x and y pair (like a tuple in python)
typedef struct Point {
	double x;
	double y;
} Point;

//takes an angle and returns the equivalent angle in the range [-180,180)
int makeCircular(double reading){
	int read = (int)reading;
	if ( read > 0 ){
		read %= 360;
		if (read < 180)
			return read;
		return read - 360;
	}
	read = (-read)%360;
	if (read < 180)
		return -read;
	return 360 - read;	
}
//returns num if min < num < max, else it returns the violated boundary. Useful for setting motor speeds
int clip(float num, int min, int max){
   if (num > max) return max;
   if (num < min) return min;
   return (int)(num);
}

//pythagorean theorem
float distanceBetween(float x1, float y1, float x2, float y2){
	return sqrt( (x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) );
}
//pythagorean theorem
float distanceBetweenPoints(Point* a, Point* b){
	return distanceBetween(a->x, a->y, b->x, b->y);
}

//calculates the angle of the point x,y
float angleOf(float x, float y){
	return atan2(y,x)*RAD_TO_DEGREE;
}
//calculates the minimum amount that (x,y) would have to rotate to have a zero dot product with (1,0)
//used when aligning with a point
float cleverAngleOf(float x, float y){
	float a = angleOf(x, y);
	if (abs(a) > 90)
		return makeCircular(a-180);
	return a;
}

Point rotatePoint(Point* p, float angle){
	float r = distanceBetween(p->x, p->y, 0, 0);
	float a = (angle + angleOf(p->x, p->y))/RAD_TO_DEGREE;
	Point ret = {.x = r*cos(a), .y = r*sin(a)};
	return ret;
}

int territory(Point* p){
	float f = angleOf(p->x, p->y);
	f = f < 0 ? f + 360 : f;
	if (f > 330){
		return 0;
	}
	return (int)(f*6/360.0 +.5);
	
}

//the angle to face to point to a wall
int wallAngle(Point* me){
	float ang = angleOf(me->x, me->y)*6/360;
	int offs = ang > 0 ? 30 : -30;
	return  ((int)ang)*60 + offs;	
}
