#include <joyos.h>
#include <math.h>
#include "dmath.h"

#define VPS_TO_US (0.0270635995)
#define ANGLE_TO_DEGREE (0.0879335613)
#define convert(x) ( x*0.0270635995 )
#define convertAngle(x) ( x*0.0879335613 )

int mvel = -255;

void calibrate(){
	pause(50);
	copy_objects();
	gyro_set_degrees(convertAngle(game.coords[0].theta));
}

//returns the reading of the gyroscope in the range [-180,180) 
float gyroCircualrRead(){
	return makeCircular(gyro_get_degrees());
} 

//the robot's heading
float getHeading(){
	//return convertAngle(game.coords[0].theta);
	return gyroCircualrRead();
}


float myX(){
	return convert(game.coords[0].x);
}

float myY(){
	return convert(game.coords[0].y);
}

float oppX(){
	return convert(game.coords[1].x);
}

float oppY(){
	return convert(game.coords[1].y);
}

float oppTheta(){
	return convert(game.coords[1].theta);
}


//Special points
//x: 39.459, y: 10.555 , theta: 23.000x: 43.464, y: -9.039 , theta: -29.000[00]
Point ORIGIN = {.x = 0., .y = .0};
Point GEAR0 = {.x = 30.793 , .y = 5.555};
Point G0Turn = {.x = 38.459, .y = 10.955 };
Point LEVER0 = {.x = 34.8 , .y = -4.03};
Point L0Pull = {.x = 44.164, .y = -8.339 };
Point HOME = {.x = 24,.y = 0};
Point DROP1, DROP2, DROP0;

Point gearLocation(int territory){
	return rotatePoint(&GEAR0, territory*60);
}


Point leverLocation(int territory){
	return rotatePoint(&LEVER0, territory*60);
}

Point gearTurnLoc(int territory){
	return rotatePoint(&G0Turn, territory*60);
}


Point leverPullLoc(int territory){
	return rotatePoint(&L0Pull, territory*60);
}

void pickSides(){
	if (myX() < 0){
		HOME.x = -28;
		mvel = 255;
	}
	DROP1 = rotatePoint(&HOME, 60);
	DROP0 = rotatePoint(&HOME, 120);
	DROP2 = rotatePoint(&HOME, 180);
	servo_set_pos(4, 480);
	servo_set_pos(5, 508);
	
}

Point* dumpSite(){
	Point me = {.x = myX(), .y = myY()};
	Point op = {.x = oppX(), .y = oppY()};
	int i = territory(&op);
	if (i == territory(&DROP0)){
		return distanceBetweenPoints(&me, &DROP1) < distanceBetweenPoints(&me, &DROP2) ? &DROP1 : &DROP2;
	};
	Point* dump = &DROP0;
	float dist = distanceBetweenPoints(&me, dump);
	if (territory(&DROP1) != i && distanceBetweenPoints(&me, &DROP1) < dist){
		dump = &DROP1;
		dist = distanceBetweenPoints(&me, &DROP1);
	}
	if (territory(&DROP2) != i && distanceBetweenPoints(&me, &DROP2) < dist){
		dump = &DROP2;
	}
	return dump;
}

bool claim(int micros){
	motor_set_vel(2, mvel);
	pause(micros);
	motor_set_vel(2, 0);
	copy_objects();
	Point me = {.x = myX(), .y = myY()};
	return game.territories[territory(&me)].owner == 13;
}

bool mine(){
	Point me = {.x = myX(), .y = myY()};
	int terr = territory(&me);
	int8_t balls = game.territories[terr].remaining;
	servo_set_pos(4, 250);
	pause(1200);
	copy_objects();
	int8_t rem = game.territories[terr].remaining;
	if (rem == balls && rem > 0){
		servo_set_pos(4, 480);
		return false;
	}
	servo_set_pos(4, 380);
	pause(400);
	while (rem < balls && rem > 0){
		balls = rem;
		servo_set_pos(4, 250);
		pause(600);
		servo_set_pos(4, 380);
		pause(400);
		copy_objects();
		rem = game.territories[terr].remaining;
	}
	servo_set_pos(4, 480);
	return true;
	
}

void releaseBalls(){	
	servo_set_pos(5, 200);
	pause(2500);
	servo_set_pos(5, 508);
}

